#include "SourceCounterMain.h"

#include "CntSettingDlg.h"
#include "SrcTypeSelDlg.h"

#include <wx/tokenzr.h>
#include <wx/msgdlg.h>
#include <wx/mimetype.h>


SourceCounterDialog::SourceCounterDialog(wxWindow* parent) : SourceCounterDialogBase(parent), m_pCountingMgr(0)
{
	m_dlgAddDir = new wxDirDialog(this, _("Select directory"), wxEmptyString, wxDD_DIR_MUST_EXIST, wxDefaultPosition, wxDefaultSize, _T("wxDirDialog"));
}

SourceCounterDialog::~SourceCounterDialog()
{
	if (m_pCountingMgr)
	{
		delete m_pCountingMgr;
		m_pCountingMgr = 0;
	}
}

void SourceCounterDialog::OnClose(wxCloseEvent& event)
{
	Destroy();
}

void SourceCounterDialog::OnQuit(wxCommandEvent& event)
{
	if (m_pCountingMgr->GetStatus() == NManagerStatusRunning)
	{
		m_pCountingMgr->SetStatus(NManagerStatusStop);
	}

	Destroy();
}


/**
 * More counting param setting
 */
void SourceCounterDialog::OnBtnMoreSettingsClick(wxCommandEvent& event)
{
	CountingParam* pCountingParam = m_pCountingMgr->GetCountingParam();
	SettingParam* pParam = &(pCountingParam->m_settingParam);

	CntSettingDlg dlg(this);
	dlg.SetSettingParam(pParam);

	if (wxID_OK == dlg.ShowModal())
	{
		dlg.GetSettingParam(pCountingParam->m_settingParam);
	}
}

void SourceCounterDialog::OnBtnAddDirClick(wxCommandEvent& event)
{
	if (wxID_OK == m_dlgAddDir->ShowModal())
	{
		wxString strPath = m_dlgAddDir->GetPath();
		int nIndex = m_lbxSrcFolder->Append(strPath);
		m_lbxSrcFolder->Check(nIndex, true);
	}
}

void SourceCounterDialog::OnBtnDeleteClick(wxCommandEvent& event)
{
	// Get selection
	int nIndex = -1;
	nIndex = m_lbxSrcFolder->GetSelection();

	if (-1 == nIndex) return;

	// Delete item
	m_lbxSrcFolder->Delete(nIndex);
}

void SourceCounterDialog::OnBtnSelSrcTypeClick(wxCommandEvent& event)
{
	SrcTypeSelDlg dlg(this);
	dlg.SetMapData(m_pCountingMgr->GetCounterRules(), m_pCountingMgr->GetFileExtensions());

	if (wxID_OK == dlg.ShowModal())
	{
		wxString strSrcTypes;
		dlg.GetSrcTypes(strSrcTypes);
		m_cmbSrcTypes->Append(strSrcTypes);
		m_cmbSrcTypes->SetValue(strSrcTypes);

	}
}

void SourceCounterDialog::OnBtnStartClick(wxCommandEvent& event)
{
	// Check source folder exist
	int nCount = m_lbxSrcFolder->GetCount();
	if (0 == nCount)
	{
		wxMessageBox(_("Please add source folder."), DIALOG_CAPTIOM, wxOK, this);
		return;
	}

	// Check source folder have been checked
	bool bHasChecked = false;
	for (int i = 0; i < nCount; i++)
	{
		bHasChecked = m_lbxSrcFolder->IsChecked(i);
		if (bHasChecked)
		{
			break;
		}
	}

	if (!bHasChecked)
	{
		wxMessageBox(_("Please check counting folder."), DIALOG_CAPTIOM);
		return;
	}

	// Check source types
	wxString strSrcTypes;
	strSrcTypes.Empty();
	strSrcTypes = m_cmbSrcTypes->GetValue();
	if (strSrcTypes.IsEmpty())
	{
		wxMessageBox(_("Please select source types."), DIALOG_CAPTIOM);
		return;
	}

	//
	// Prepare counting
	//

	m_pCountingMgr->Clear();	    // Clear prev counting info

	m_pCountingMgr->SetStatus(NManagerStatusRunning);

	m_lstResult->DeleteAllItems();

	// Initial labels
	wxString strTemp(_T("0"));
	m_lblTotalFiles->SetLabel(strTemp);
	m_lblTotalSize->SetLabel(strTemp);
	m_lblTotalLines->SetLabel(strTemp);
	m_lblCodeLines->SetLabel(strTemp);
	m_lblCommentLines->SetLabel(strTemp);
	m_lblBlankLines->SetLabel(strTemp);

	//
	// Collection of counting parameter information
	//

	CountingParam countingParam;

	// Collect source folder path for counting
	for (int i = 0; i < nCount; i++)
	{
		if (m_lbxSrcFolder->IsChecked(i))
		{
			countingParam.m_arrSrcFolderPath.Add(m_lbxSrcFolder->GetString(i));
		}
	}

	// Collect source types
	wxStringTokenizer tkz(strSrcTypes, _T(";"));
	while (tkz.HasMoreTokens())
	{
		wxString token = tkz.GetNextToken();
		countingParam.m_arrSrcType.Add(token);
	}

	// Collect recursive counting
	countingParam.m_bRecursiveCounting = m_chbRecursiveCouting->GetValue();
	// TODO: countingParam.m_settingParam.m_nCountingMethodType = ;

	// Set counting parameter to CountingManager
	m_pCountingMgr->SetCountingParam(&countingParam);

	//
	// Start counting by call CountingManager
	//

	// Attach observer
	m_pCountingMgr->AttachObserver(this);
	m_pCountingMgr->StartCounting();
}


void SourceCounterDialog::UpdateCountingInfoCtrls()
{
	// Get countingInfo
	CountingInfo* pCountingInfo = m_pCountingMgr->GetCountingInfo();
	CountingFileInfo* pCountingFileInfo = pCountingInfo->GetLastCountingFileInfo();

	//
	// Insert item to resutl listctrl
	//
	long nIndex = m_lstResult->InsertItem(m_lstResult->GetItemCount(), pCountingFileInfo->m_strFileName);
	m_lstResult->SetItem(nIndex, 1, pCountingFileInfo->m_strFileExtName);

	wxString strTemp;

	strTemp.Printf(_T("%d"), pCountingFileInfo->m_nTotalStatement);
	m_lstResult->SetItem(nIndex, 2, strTemp);

	strTemp.Printf(_T("%d"), pCountingFileInfo->m_nCodeStatement);
	m_lstResult->SetItem(nIndex, 3, strTemp);

	strTemp.Printf(_T("%d"), pCountingFileInfo->m_nCommentStatement);
	m_lstResult->SetItem(nIndex, 4, strTemp);

	strTemp.Printf(_T("%d"), pCountingFileInfo->m_nBlankStatement);
	m_lstResult->SetItem(nIndex, 5, strTemp);

	strTemp.Printf(_T("%d"), pCountingFileInfo->m_nSize);
	m_lstResult->SetItem(nIndex, 6, strTemp);

	m_lstResult->SetItem(nIndex, 7, pCountingFileInfo->m_strFolderPath);

	// Set data to item
	m_lstResult->SetItemData(nIndex, (long)pCountingFileInfo);

	// Ensure visible
	if (nIndex % 3 == 0) m_lstResult->EnsureVisible(nIndex);

	///////////////////////////////////////////////////////////////////
	// Update labels
	strTemp.Printf(_T("%d"), pCountingInfo->m_nTotalFile);
	m_lblTotalFiles->SetLabel(strTemp);

	strTemp.Printf(_T("%d"), pCountingInfo->m_nTotalSize);
	m_lblTotalSize->SetLabel(strTemp);

	strTemp.Printf(_T("%d"), pCountingInfo->m_nTotalStatement);
	m_lblTotalLines->SetLabel(strTemp);

	strTemp.Printf(_T("%d"), pCountingInfo->m_nTotalCodeStatement);
	m_lblCodeLines->SetLabel(strTemp);

	strTemp.Printf(_T("%d"), pCountingInfo->m_nTotalCommentStatement);
	m_lblCommentLines->SetLabel(strTemp);

	strTemp.Printf(_T("%d"), pCountingInfo->m_nTotalBlankStatement);
	m_lblBlankLines->SetLabel(strTemp);
}

void SourceCounterDialog::OnInit(wxInitDialogEvent& event)
{
	if (!m_pCountingMgr) m_pCountingMgr = new CountingManager();

	m_pCountingMgr->loadRules();

	m_lstResult->InsertColumn(0, _("File"));
	m_lstResult->InsertColumn(1, _("Type"));
	m_lstResult->InsertColumn(2, _("Lines"));
	m_lstResult->InsertColumn(3, _("Code lines"));
	m_lstResult->InsertColumn(4, _("Comment lines"));
	m_lstResult->InsertColumn(5, _("Blank lines"));
	m_lstResult->InsertColumn(6, _("Size"));
	m_lstResult->InsertColumn(7, _("Folder"));

	m_lstResult->SetColumnWidth(7, 500);
}

void SourceCounterDialog::OnBtnStopClick(wxCommandEvent& event)
{
	m_pCountingMgr->StopCounting();
}

