#include "SrcTypeSelDlg.h"
#include "CounterRuleDlg.h"

SrcTypeSelDlg::SrcTypeSelDlg(wxWindow* parent, wxWindowID id, const wxPoint& pos, const wxSize& size) :
	SrcTypeSelDlgBase(parent),
	m_pMapRule(0),
	m_pMapFileExtension(0)
{
}

SrcTypeSelDlg::~SrcTypeSelDlg()
{
}

void SrcTypeSelDlg::OnInit(wxInitDialogEvent& event)
{
	updateSrcTypeLbx();
}

void SrcTypeSelDlg::updateSrcTypeLbx()
{
	int nCnt = m_lbxSrcTypes->GetCount();
	if (nCnt > 0) m_lbxSrcTypes->Clear();

	MapStrToFileExtension::iterator itFileExt;
	FileExtension* pFileExt = 0;
	int nIndex = 0;
	for (itFileExt = m_pMapFileExtension->begin(); itFileExt != m_pMapFileExtension->end(); ++itFileExt)
	{
		pFileExt = itFileExt->second;
		nIndex = m_lbxSrcTypes->Append(pFileExt->m_strName + _T(" - ") + pFileExt->m_strDesc + _T(" ( ") + pFileExt->m_strCounterType + _T(" style )"));

		if (pFileExt->m_bSel) m_lbxSrcTypes->Check(nIndex);
	}
}

void SrcTypeSelDlg::OnBtnOkClick(wxCommandEvent& event)
{
	//
	// UnSelect fileExt obj
	//
	MapStrToFileExtension::iterator it;
	FileExtension* pExt = 0;
	for (it = m_pMapFileExtension->begin(); it != m_pMapFileExtension->end(); ++it)
	{
		pExt = it->second;
		if (pExt) pExt->m_bSel = false;
	}

	//
	// Build return src types string
	//

	int nNum = m_lbxSrcTypes->GetCount();
	wxString strTemp;
	for (int i = 0; i < nNum; i++)
	{
		//
		if (m_lbxSrcTypes->IsChecked(i))
		{
			strTemp = m_lbxSrcTypes->GetString(i);
			int index = strTemp.Find(_T('-'));
			strTemp = strTemp.Left(index - 1); // .cpp - : .cpp
			//
			m_strSrcTypes = m_strSrcTypes + strTemp + _T(";");

			//
			// Set pFileExt m_bSel to true
			//
			it = m_pMapFileExtension->find(strTemp);
			if (it != m_pMapFileExtension->end())
			{
				pExt = it->second;
				pExt->m_bSel = true;
			}
		}
	}

	EndModal(wxID_OK);
}

void SrcTypeSelDlg::OnBtnCancelClick(wxCommandEvent& event)
{
	EndModal(wxID_CANCEL);
}

void SrcTypeSelDlg::Onm_ckbSelAllClick(wxCommandEvent& event)
{
	bool bChecked = m_ckbSelAll->IsChecked();
	int nNum = m_lbxSrcTypes->GetCount();

	for (int i = 0; i < nNum; i++)
	{
		m_lbxSrcTypes->Check(i, bChecked);
	}
}

void SrcTypeSelDlg::OnLbxSrcTypesSelect(wxCommandEvent& event)
{
	updateButtons();
}

void SrcTypeSelDlg::updateButtons()
{
	bool bEnable = false;
	if (wxNOT_FOUND != m_lbxSrcTypes->GetSelection())
	{
		bEnable = true;
	}

	m_btnEdit->Enable(bEnable);
	m_btnDel->Enable(bEnable);
}

void SrcTypeSelDlg::OnBtnAddClick(wxCommandEvent& event)
{
	wxString strTemp = _T("tmp");
	FileExtension* pFileExt = new FileExtension(strTemp, strTemp, strTemp);

	CounterRuleDlg dlg(this);
	dlg.SetRuleData(m_pMapRule, pFileExt);

	if (wxID_OK == dlg.ShowModal())
	{
		//
		// Check exist
		//
		MapStrToFileExtension::iterator it;
		it = m_pMapFileExtension->find(pFileExt->m_strName);
		if (it != m_pMapFileExtension->end())
		{
			wxMessageBox(_("Extension has existed!"));
			return;
		}

		//
		// Add pFileExt obj to FileExtMap
		//
		(*m_pMapFileExtension)[pFileExt->m_strName] = pFileExt;

		//
		updateSrcTypeLbx();
	}
	else
	{
		// Delete pFIleExt obj
		delete pFileExt;
	}
}

FileExtension* SrcTypeSelDlg::getSelItemData()
{
	// Get selected rule
	int nNum = m_lbxSrcTypes->GetCount();

	wxString strTemp;
	for (int i = 0; i < nNum; i++)
	{
		//
		if (m_lbxSrcTypes->IsSelected(i))
		{
			strTemp = m_lbxSrcTypes->GetString(i);
			int index = strTemp.Find(_T('-'));
			strTemp = strTemp.Left(index - 1); // .cpp - : .cpp
		}
	}

	// Find file extension in map
	MapStrToFileExtension::iterator itFileExt;
	FileExtension* pFileExt = 0;

	for (itFileExt = m_pMapFileExtension->begin(); itFileExt != m_pMapFileExtension->end(); ++itFileExt)
	{
		pFileExt = itFileExt->second;
		if (strTemp.CmpNoCase(pFileExt->m_strName) == 0)
			break;
	}

	return pFileExt;
}

void SrcTypeSelDlg::OnBtnEditClick(wxCommandEvent& event)
{
	FileExtension* pFileExt = getSelItemData();
	if (!pFileExt)
		wxMessageBox(_T("Error!"));

	CounterRuleDlg dlg(this, CounterRuleDlg::RuleModeEdit);
	dlg.SetRuleData(m_pMapRule, pFileExt);

	if (wxID_OK == dlg.ShowModal())
	{
		// Refresh listbox content
		updateSrcTypeLbx();
	}
}

void SrcTypeSelDlg::OnBtnDelClick(wxCommandEvent& event)
{
	FileExtension* pFileExt = getSelItemData();
	if (!pFileExt) wxMessageBox(_T("Error!"));

	m_pMapFileExtension->erase(pFileExt->m_strName);

	delete pFileExt;
	pFileExt = 0;

	updateSrcTypeLbx();
}
